@@ -65,6 +65,8 @@ arv_buffer_new_full (size_t size, void *preallocated, void *user_data, GDestroyN
 	buffer->priv->size = size;
 	buffer->priv->user_data = user_data;
 	buffer->priv->user_data_destroy_func = user_data_destroy_func;
+	buffer->priv->chunk_endianness = G_BIG_ENDIAN;
+	buffer->priv->payload_type = ARV_BUFFER_PAYLOAD_TYPE_UNKNOWN;
 
 	if (preallocated != NULL) {
 		buffer->priv->is_preallocated = TRUE;
@@ -117,23 +119,6 @@ arv_buffer_new_allocate (size_t size)
 	return arv_buffer_new_full (size, NULL, NULL, NULL);
 }
 
-/**
- * arv_buffer_clear:
- * @buffer: a #ArvBuffer
- *
- * Clears the buffer status.
- *
- * Since: 0.2.0
- */
-
-void
-arv_buffer_clear (ArvBuffer *buffer)
-{
-	g_return_if_fail (ARV_IS_BUFFER (buffer));
-
-	buffer->priv->status = ARV_BUFFER_STATUS_CLEARED;
-}
-
 /**
  * arv_buffer_get_data:
  * @buffer: a #ArvBuffer
@@ -187,7 +172,8 @@ arv_buffer_get_chunk_data (ArvBuffer *buffer, guint64 chunk_id, size_t *size)
 
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), NULL);
 	g_return_val_if_fail (buffer->priv->data != NULL, NULL);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_CHUNK_DATA, NULL);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_CHUNK_DATA ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, NULL);
 
 	if (buffer->priv->status != ARV_BUFFER_STATUS_SUCCESS)
 		return NULL;
@@ -195,20 +181,32 @@ arv_buffer_get_chunk_data (ArvBuffer *buffer, guint64 chunk_id, size_t *size)
 	data = buffer->priv->data;
 	offset = buffer->priv->size - sizeof (ArvChunkInfos);
 	while (offset > 0) {
+		guint32 id;
+		guint32 chunk_size;
+
 		infos = (ArvChunkInfos *) &data[offset];
-		if (GUINT32_FROM_BE (infos->id) == chunk_id) {
+
+		if (buffer->priv->chunk_endianness == G_BIG_ENDIAN) {
+			id = GUINT32_FROM_BE (infos->id);
+			chunk_size = GUINT32_FROM_BE (infos->size);
+		} else {
+			id = GUINT32_FROM_LE (infos->id);
+			chunk_size = GUINT32_FROM_LE (infos->size);
+		}
+
+		if (id == chunk_id) {
 			ptrdiff_t data_offset;
 
-			data_offset = offset - GUINT32_FROM_BE (infos->size);
+			data_offset = offset - chunk_size;
 			if (data_offset >= 0) {
 				if (size != NULL)
-					*size = GUINT32_FROM_BE (infos->size);
+					*size = chunk_size;
 				return &data[data_offset];
 			} else
 		       		return NULL;
 		}
-		if (GUINT32_FROM_BE (infos->size) > 0)
-			offset = offset - GUINT32_FROM_BE (infos->size) - sizeof (ArvChunkInfos);
+		if (chunk_size > 0)
+			offset = offset - chunk_size - sizeof (ArvChunkInfos);
 		else
 			offset = 0;
 	};
@@ -270,28 +268,7 @@ arv_buffer_get_payload_type (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), -1);
 
-	switch (buffer->priv->gvsp_payload_type) {
-		case ARV_GVSP_PAYLOAD_TYPE_IMAGE:
-			return ARV_BUFFER_PAYLOAD_TYPE_IMAGE;
-		case ARV_GVSP_PAYLOAD_TYPE_RAWDATA:
-			return ARV_BUFFER_PAYLOAD_TYPE_RAWDATA;
-		case ARV_GVSP_PAYLOAD_TYPE_FILE:
-			return ARV_BUFFER_PAYLOAD_TYPE_FILE;
-		case ARV_GVSP_PAYLOAD_TYPE_CHUNK_DATA:
-			return ARV_BUFFER_PAYLOAD_TYPE_CHUNK_DATA;
-		case ARV_GVSP_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA:
-			return ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA;
-		case ARV_GVSP_PAYLOAD_TYPE_JPEG:
-			return ARV_BUFFER_PAYLOAD_TYPE_JPEG;
-		case ARV_GVSP_PAYLOAD_TYPE_JPEG2000:
-			return ARV_BUFFER_PAYLOAD_TYPE_JPEG2000;
-		case ARV_GVSP_PAYLOAD_TYPE_H264:
-			return ARV_BUFFER_PAYLOAD_TYPE_H264;
-		case ARV_GVSP_PAYLOAD_TYPE_MULTIZONE_IMAGE:
-			return ARV_BUFFER_PAYLOAD_TYPE_MULTIZONE_IMAGE;
-		default:
-			return ARV_BUFFER_PAYLOAD_TYPE_UNKNOWN;
-	}
+	return buffer->priv->payload_type;
 }
 
 /**
@@ -300,7 +277,8 @@ arv_buffer_get_payload_type (ArvBuffer *buffer)
  *
  * Gets the buffer camera timestamp, expressed as nanoseconds. Not all devices
  * provide reliable timestamp, which means sometimes its better to rely on the
- * buffer completion host local time (given by @g_get_realtime for example).
+ * buffer completion host local time, or to use
+ * arv_buffer_get_system_timestamp().
  *
  * Returns: buffer timestamp, in nanoseconds.
  *
@@ -334,6 +312,46 @@ arv_buffer_set_timestamp (ArvBuffer *buffer, guint64 timestamp_ns)
 	buffer->priv->timestamp_ns = timestamp_ns;
 }
 
+/**
+ * arv_buffer_get_system_timestamp:
+ * @buffer: a #ArvBuffer
+ *
+ * Gets the system timestamp for when the frame was received. Expressed in
+ * nanoseconds.
+ *
+ * Returns: buffer system timestamp, in nanoseconds.
+ *
+ * Since: 0.6.0
+ */
+
+guint64
+arv_buffer_get_system_timestamp (ArvBuffer *buffer)
+{
+	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
+
+	return buffer->priv->system_timestamp_ns;
+}
+
+/**
+ * arv_buffer_set_system_timestamp:
+ * @buffer: a #ArvBuffer
+ * @timestamp_ns: a timestamp, expressed as nanoseconds
+ *
+ * Sets the system timestamp for when the frame was received. Expressed in
+ * nanoseconds.
+ *
+ * Since: 0.6.0
+ */
+
+void
+arv_buffer_set_system_timestamp (ArvBuffer *buffer, guint64 timestamp_ns)
+{
+	g_return_if_fail (ARV_IS_BUFFER (buffer));
+
+	buffer->priv->system_timestamp_ns = timestamp_ns;
+}
+
+
 /**
  * arv_buffer_get_frame_id:
  * @buffer: a #ArvBuffer
@@ -371,7 +389,8 @@ void
 arv_buffer_get_image_region (ArvBuffer *buffer, gint *x, gint *y, gint *width, gint *height)
 {
 	g_return_if_fail (ARV_IS_BUFFER (buffer));
-	g_return_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE);
+	g_return_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			  buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA);
 
 	if (x != NULL)
 		*x = buffer->priv->x_offset;
@@ -398,7 +417,8 @@ gint
 arv_buffer_get_image_width (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE, 0);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, 0);
 
 	return buffer->priv->width;
 }
@@ -418,7 +438,8 @@ gint
 arv_buffer_get_image_height (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE, 0);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, 0);
 
 	return buffer->priv->height;
 }
@@ -438,7 +459,8 @@ gint
 arv_buffer_get_image_x (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE, 0);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, 0);
 
 	return buffer->priv->x_offset;
 }
@@ -458,7 +480,8 @@ gint
 arv_buffer_get_image_y (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE, 0);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, 0);
 
 	return buffer->priv->y_offset;
 }
@@ -478,7 +501,8 @@ ArvPixelFormat
 arv_buffer_get_image_pixel_format (ArvBuffer *buffer)
 {
 	g_return_val_if_fail (ARV_IS_BUFFER (buffer), 0);
-	g_return_val_if_fail (buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE, 0);
+	g_return_val_if_fail (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+			      buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA, 0);
 
 	return buffer->priv->pixel_format;
 }
@@ -512,11 +536,17 @@ arv_buffer_class_init (ArvBufferClass *this_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (this_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (this_class, sizeof (ArvBufferPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (this_class);
 
 	object_class->finalize = arv_buffer_finalize;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvBuffer, arv_buffer, G_TYPE_OBJECT)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvBuffer, arv_buffer, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvBuffer))
+#endif
