@@ -68,6 +68,7 @@ static const char *arv_evaluator_status_strings[] = {
 	"remaining operands",
 	"division by zero",
 	"stack overflow",
+	"invalid double function",
 	"forbidden recursion"
 };
 
@@ -197,6 +198,7 @@ static ArvEvaluatorTokenInfos arv_evaluator_token_infos[] = {
 
 typedef struct {
 	ArvEvaluatorTokenId	token_id;
+	gint32 parenthesis_level;
 	union {
 		double		v_double;
 		gint64		v_int64;
@@ -204,6 +206,11 @@ typedef struct {
 	} data;
 } ArvEvaluatorToken;
 
+typedef struct {
+	gint32 parenthesis_level;
+	ArvValue value;
+} ArvEvaluatorValuesStackItem;
+
 static ArvEvaluatorToken *
 arv_evaluator_token_new (ArvEvaluatorTokenId token_id)
 {
@@ -508,13 +515,33 @@ arv_get_next_token (char **expression, gboolean previous_token_was_operand, gboo
 	return token;
 }
 
+static double
+round_with_precision (double x, gint64 precision)
+{
+	double pow_multiplier = pow(10, precision);
+	return (round (x*pow_multiplier))/pow_multiplier;
+}
+
+static int
+get_arguments_count(ArvEvaluatorValuesStackItem * stack, int current_index)
+{
+	int arguments_count = 0;
+	int index = current_index;
+	int parenthesis_level = stack[index].parenthesis_level;
+	while ((index>=0) && (stack[index].parenthesis_level>=parenthesis_level)){
+		index--;
+		arguments_count++;
+	}
+	return arguments_count;
+}
+
 static ArvEvaluatorStatus
 evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v_double)
 {
 	ArvEvaluatorToken *token;
 	ArvEvaluatorStatus status;
 	GSList *iter;
-	ArvValue stack[ARV_EVALUATOR_STACK_SIZE];
+	ArvEvaluatorValuesStackItem stack[ARV_EVALUATOR_STACK_SIZE];
 	ArvValue *value;
 	int index = -1;
 	gboolean integer_mode;
@@ -543,297 +570,356 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 
 		arv_evaluator_token_debug (token, variables);
 
+		int actual_arguments_count = arv_evaluator_token_infos[token->token_id].n_args;
+
 		switch (token->token_id) {
 			case ARV_EVALUATOR_TOKEN_LOGICAL_AND:
-				arv_value_set_int64 (&stack[index-1],
-						      arv_value_get_int64 (&stack[index-1]) &&
-						      arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						      arv_value_get_int64 (&stack[index-1].value) &&
+						      arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_LOGICAL_OR:
-				arv_value_set_int64 (&stack[index-1],
-						      arv_value_get_int64 (&stack[index-1]) ||
-						      arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						      arv_value_get_int64 (&stack[index-1].value) ||
+						      arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_BITWISE_NOT:
-				arv_value_set_int64 (&stack[index],
-						      ~arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index].value,
+						      ~arv_value_get_int64 (&stack[index].value));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_BITWISE_AND:
-				arv_value_set_int64 (&stack[index-1],
-						      arv_value_get_int64 (&stack[index-1]) &
-						      arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						      arv_value_get_int64 (&stack[index-1].value) &
+						      arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_BITWISE_OR:
-				arv_value_set_int64 (&stack[index-1],
-						      arv_value_get_int64 (&stack[index-1]) |
-						      arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						      arv_value_get_int64 (&stack[index-1].value) |
+						      arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_BITWISE_XOR:
-				arv_value_set_int64 (&stack[index-1],
-						      arv_value_get_int64 (&stack[index-1]) ^
-						      arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						      arv_value_get_int64 (&stack[index-1].value) ^
+						      arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_EQUAL:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) ==
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) ==
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) ==
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) ==
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_NOT_EQUAL:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) !=
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) !=
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) !=
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) !=
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_LESS_OR_EQUAL:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) <=
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) <=
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) <=
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) <=
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_GREATER_OR_EQUAL:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) >=
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) >=
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) >=
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) >=
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_LESS:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) <
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) <
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) <
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) <
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_GREATER:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) >
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) >
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_int64 (&stack[index - 1],
-							     arv_value_get_double (&stack[index-1]) >
-							     arv_value_get_double (&stack[index]));
+					arv_value_set_int64 (&stack[index - 1].value,
+							     arv_value_get_double (&stack[index-1].value) >
+							     arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_SHIFT_RIGHT:
-				arv_value_set_int64 (&stack[index-1],
-						     arv_value_get_int64 (&stack[index-1]) >>
-						     arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						     arv_value_get_int64 (&stack[index-1].value) >>
+						     arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_SHIFT_LEFT:
-				arv_value_set_int64 (&stack[index-1],
-						     arv_value_get_int64 (&stack[index-1]) <<
-						     arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						     arv_value_get_int64 (&stack[index-1].value) <<
+						     arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_SUBSTRACTION:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) -
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) -
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_double (&stack[index-1],
-							      arv_value_get_double (&stack[index-1]) -
-							      arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index-1].value,
+							      arv_value_get_double (&stack[index-1].value) -
+							      arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_ADDITION:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) +
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) +
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_double (&stack[index-1],
-							      arv_value_get_double (&stack[index-1]) +
-							      arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index-1].value,
+							      arv_value_get_double (&stack[index-1].value) +
+							      arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_REMAINDER:
-				if (arv_value_get_int64 (&stack[index]) == 0) {
+				if (arv_value_get_int64 (&stack[index].value) == 0) {
 					status = ARV_EVALUATOR_STATUS_DIVISION_BY_ZERO;
 					goto CLEANUP;
 				}
-				arv_value_set_int64 (&stack[index-1],
-						     arv_value_get_int64 (&stack[index-1]) %
-						     arv_value_get_int64 (&stack[index]));
+				arv_value_set_int64 (&stack[index-1].value,
+						     arv_value_get_int64 (&stack[index-1].value) %
+						     arv_value_get_int64 (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_DIVISION:
 				if (integer_mode) {
-					if (arv_value_get_int64 (&stack[index]) == 0) {
+					if (arv_value_get_int64 (&stack[index].value) == 0) {
 						status = ARV_EVALUATOR_STATUS_DIVISION_BY_ZERO;
 						goto CLEANUP;
 					}
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) /
-							     arv_value_get_int64 (&stack[index]));
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) /
+							     arv_value_get_int64 (&stack[index].value));
 				} else {
-					if (arv_value_get_double (&stack[index]) == 0.0) {
+					if (arv_value_get_double (&stack[index].value) == 0.0) {
 						status = ARV_EVALUATOR_STATUS_DIVISION_BY_ZERO;
 						goto CLEANUP;
 					}
-					arv_value_set_double (&stack[index-1],
-							      arv_value_get_double (&stack[index-1]) /
-							      arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index-1].value,
+							      arv_value_get_double (&stack[index-1].value) /
+							      arv_value_get_double (&stack[index].value));
 				}
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_MULTIPLICATION:
 				if (integer_mode ||
-				    (arv_value_holds_int64 (&stack[index-1]) &&
-				     arv_value_holds_int64 (&stack[index])))
-					arv_value_set_int64 (&stack[index-1],
-							     arv_value_get_int64 (&stack[index-1]) *
-							     arv_value_get_int64 (&stack[index]));
+				    (arv_value_holds_int64 (&stack[index-1].value) &&
+				     arv_value_holds_int64 (&stack[index].value)))
+					arv_value_set_int64 (&stack[index-1].value,
+							     arv_value_get_int64 (&stack[index-1].value) *
+							     arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_double (&stack[index-1],
-							      arv_value_get_double (&stack[index-1]) *
-							      arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index-1].value,
+							      arv_value_get_double (&stack[index-1].value) *
+							      arv_value_get_double (&stack[index].value));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_POWER:
 				if (integer_mode)
-					arv_value_set_int64 (&stack[index-1],
-							     pow (arv_value_get_int64(&stack[index-1]),
-								  arv_value_get_int64(&stack[index])));
+					arv_value_set_int64 (&stack[index-1].value,
+							     pow (arv_value_get_int64(&stack[index-1].value),
+								  arv_value_get_int64(&stack[index].value)));
 				else
-					arv_value_set_double (&stack[index-1],
-							      pow (arv_value_get_double(&stack[index-1]),
-								   arv_value_get_double(&stack[index])));
+					arv_value_set_double (&stack[index-1].value,
+							      pow (arv_value_get_double(&stack[index-1].value),
+								   arv_value_get_double(&stack[index].value)));
+				stack[index-1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_MINUS:
-				if (integer_mode || arv_value_holds_int64 (&stack[index]))
-					arv_value_set_int64 (&stack[index],
-							     -arv_value_get_int64 (&stack[index]));
+				if (integer_mode || arv_value_holds_int64 (&stack[index].value))
+					arv_value_set_int64 (&stack[index].value,
+							     -arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_double (&stack[index],
-							      -arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index].value,
+							      -arv_value_get_double (&stack[index].value));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_PLUS:
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_SIN:
-				arv_value_set_double (&stack[index], sin (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, sin (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_COS:
-				arv_value_set_double (&stack[index], cos (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, cos (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_SGN:
-				if (integer_mode || arv_value_holds_int64 (&stack[index])) {
-					gint64 int_value = arv_value_get_int64 (&stack[index]);
+				if (integer_mode || arv_value_holds_int64 (&stack[index].value)) {
+					gint64 int_value = arv_value_get_int64 (&stack[index].value);
 					if (int_value < 0)
-						arv_value_set_int64 (&stack[index], -1);
+						arv_value_set_int64 (&stack[index].value, -1);
 					else if (int_value > 0)
-						arv_value_set_int64 (&stack[index], 1);
+						arv_value_set_int64 (&stack[index].value, 1);
 					else
-						arv_value_set_int64 (&stack[index], 0);
+						arv_value_set_int64 (&stack[index].value, 0);
 				} else {
-					double dbl_value = arv_value_get_double (&stack[index]);
+					double dbl_value = arv_value_get_double (&stack[index].value);
 					if (dbl_value < 0.0)
-						arv_value_set_int64 (&stack[index], -1);
+						arv_value_set_int64 (&stack[index].value, -1);
 					else if (dbl_value > 0.0)
-						arv_value_set_int64 (&stack[index], 1);
+						arv_value_set_int64 (&stack[index].value, 1);
 					else
-						arv_value_set_int64 (&stack[index], 0);
+						arv_value_set_int64 (&stack[index].value, 0);
 				}
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_NEG:
-				if (integer_mode || arv_value_holds_int64 (&stack[index]))
-					arv_value_set_int64 (&stack[index],
-							     -arv_value_get_int64 (&stack[index]));
+				if (integer_mode || arv_value_holds_int64 (&stack[index].value))
+					arv_value_set_int64 (&stack[index].value,
+							     -arv_value_get_int64 (&stack[index].value));
 				else
-					arv_value_set_double (&stack[index],
-							      -arv_value_get_double (&stack[index]));
+					arv_value_set_double (&stack[index].value,
+							      -arv_value_get_double (&stack[index].value));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ATAN:
-				arv_value_set_double (&stack[index], atan (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, atan (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_TAN:
-				arv_value_set_double (&stack[index], tan (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, tan (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ABS:
-				if (arv_value_holds_double (&stack[index]))
-					arv_value_set_double (&stack[index],
-							      fabs (arv_value_get_double (&stack[index])));
+				if (arv_value_holds_double (&stack[index].value))
+					arv_value_set_double (&stack[index].value,
+							      fabs (arv_value_get_double (&stack[index].value)));
 				else
-					arv_value_set_int64 (&stack[index],
-							     labs (arv_value_get_int64 (&stack[index])));
+					arv_value_set_int64 (&stack[index].value,
+							     llabs (arv_value_get_int64 (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_EXP:
-				arv_value_set_double (&stack[index], exp (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, exp (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_LN:
-				arv_value_set_double (&stack[index], log (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, log (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_LG:
-				arv_value_set_double (&stack[index], log10 (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, log10 (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_SQRT:
-				arv_value_set_double (&stack[index], sqrt (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, sqrt (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_TRUNC:
-				if (arv_value_get_double (&stack[index]) > 0.0)
-					arv_value_set_double (&stack[index],
-							      floor (arv_value_get_double (&stack[index])));
+				if (arv_value_get_double (&stack[index].value) > 0.0)
+					arv_value_set_double (&stack[index].value,
+							      floor (arv_value_get_double (&stack[index].value)));
 				else
-					arv_value_set_double (&stack[index],
-							      ceil (arv_value_get_double (&stack[index])));
+					arv_value_set_double (&stack[index].value,
+							      ceil (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ROUND:
-				arv_value_set_double (&stack[index], round (arv_value_get_double (&stack[index])));
+				actual_arguments_count = get_arguments_count(stack, index);
+				if (actual_arguments_count==1) {
+					arv_value_set_double(&stack[index].value, round(arv_value_get_double(&stack[index].value)));
+					stack[index].parenthesis_level = token->parenthesis_level;
+				} else if (actual_arguments_count==2) {
+					arv_value_set_double(&stack[index - 1].value,
+										 round_with_precision(arv_value_get_double(&stack[index - 1].value),
+															  arv_value_get_int64(&stack[index].value)));
+					stack[index - 1].parenthesis_level = token->parenthesis_level;
+				} else {
+					if (actual_arguments_count<1) {
+						status = ARV_EVALUATOR_STATUS_MISSING_ARGUMENTS;
+					} else {
+						status = ARV_EVALUATOR_STATUS_REMAINING_OPERANDS;
+					}
+					goto CLEANUP;
+				}
+
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_FLOOR:
-				arv_value_set_double (&stack[index], floor (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, floor (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_CEIL:
-				arv_value_set_double (&stack[index], ceil (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, ceil (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ASIN:
-				arv_value_set_double (&stack[index], asin (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, asin (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ACOS:
-				arv_value_set_double (&stack[index], acos (arv_value_get_double (&stack[index])));
+				arv_value_set_double (&stack[index].value, acos (arv_value_get_double (&stack[index].value)));
+				stack[index].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_CONSTANT_INT64:
-				arv_value_set_int64 (&stack[index+1], token->data.v_int64);
+				arv_value_set_int64 (&stack[index+1].value, token->data.v_int64);
+				stack[index+1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_CONSTANT_DOUBLE:
 				if (integer_mode)
-					arv_value_set_int64 (&stack[index+1], token->data.v_double);
+					arv_value_set_int64 (&stack[index+1].value, token->data.v_double);
 				else
-					arv_value_set_double (&stack[index+1], token->data.v_double);
+					arv_value_set_double (&stack[index+1].value, token->data.v_double);
+				stack[index+1].parenthesis_level = token->parenthesis_level;
 				break;
 			case ARV_EVALUATOR_TOKEN_VARIABLE:
 				value = g_hash_table_lookup (variables, token->data.name);
 				if (value != NULL) {
-					arv_value_copy (&stack[index+1], value);
+					arv_value_copy (&stack[index+1].value, value);
+					stack[index+1].parenthesis_level = token->parenthesis_level;
 				} else {
 					status = ARV_EVALUATOR_STATUS_UNKNOWN_VARIABLE;
 					goto CLEANUP;
@@ -842,18 +928,20 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 			case ARV_EVALUATOR_TOKEN_TERNARY_COLON:
 				break;
 			case ARV_EVALUATOR_TOKEN_TERNARY_QUESTION_MARK:
-				if (arv_value_get_int64 (&stack[index-2]) != 0)
-					arv_value_copy (&stack[index-2], &stack[index-1]);
-				else
-					arv_value_copy (&stack[index-2], &stack[index]);
+				if (arv_value_get_int64 (&stack[index-2].value) != 0) {
+					arv_value_copy(&stack[index - 2].value, &stack[index - 1].value);
+					stack[index - 2].parenthesis_level = stack[index - 1].parenthesis_level;
+				} else {
+					arv_value_copy(&stack[index - 2].value, &stack[index].value);
+					stack[index - 2].parenthesis_level = stack[index].parenthesis_level;
+				}
 				break;
 			default:
 				status = ARV_EVALUATOR_STATUS_UNKNOWN_OPERATOR;
 				goto CLEANUP;
 				break;
 		}
-
-		index = index - arv_evaluator_token_infos[token->token_id].n_args + 1;
+		index = index - actual_arguments_count + 1;
 	}
 
 	if (index != 0) {
@@ -862,15 +950,15 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 	}
 
 	if (v_double != NULL)
-		*v_double = arv_value_get_double (stack);
+		*v_double = arv_value_get_double (&stack[0].value);
 
 	if (v_int64 != NULL)
-		*v_int64 = arv_value_get_int64 (stack);
+		*v_int64 = arv_value_get_int64 (&stack[0].value);
 
-	if (arv_value_holds_int64 (stack))
-		arv_log_evaluator ("[Evaluator::evaluate] Result = (int64) %Ld", arv_value_get_int64 (stack));
+	if (arv_value_holds_int64 (&stack[0].value))
+		arv_log_evaluator ("[Evaluator::evaluate] Result = (int64) %Ld", arv_value_get_int64 (&stack[0].value));
 	else
-		arv_log_evaluator ("[Evaluator::evaluate] Result = (double) %g", arv_value_get_double (stack));
+		arv_log_evaluator ("[Evaluator::evaluate] Result = (double) %g", arv_value_get_double (&stack[0].value));
 
 	return ARV_EVALUATOR_STATUS_SUCCESS;
 CLEANUP:
@@ -899,7 +987,7 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 	ArvEvaluatorToken *token;
 	ArvEvaluatorStatus status;
 	gboolean token_found;
-
+	gint32 current_parenthesis_level = 0;
 	if (expression == NULL)
 		return ARV_EVALUATOR_STATUS_EMPTY_EXPRESSION;
 
@@ -908,9 +996,10 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 
 	do {
 		token = arv_get_next_token (&expression, state->previous_token_was_operand, state->previous_token_was_right_parenthesis);
+
 		if (token != NULL) {
 			token_found = TRUE;
-
+			token->parenthesis_level = current_parenthesis_level;
 			state->previous_token_was_operand = arv_evaluator_token_is_operand (token);
 			state->previous_token_was_right_parenthesis = arv_evaluator_token_is_right_parenthesis (token);
 
@@ -983,8 +1072,10 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 				}
 				state->operator_stack = g_slist_prepend (state->operator_stack, token);
 			} else if (arv_evaluator_token_is_left_parenthesis (token)) {
+				current_parenthesis_level+=1;
 				state->operator_stack = g_slist_prepend (state->operator_stack, token);
 			} else if (arv_evaluator_token_is_right_parenthesis (token)) {
+				current_parenthesis_level-=1;
 				while (state->operator_stack != NULL &&
 				       !arv_evaluator_token_is_left_parenthesis (state->operator_stack->data)) {
 					state->token_stack = g_slist_prepend (state->token_stack, state->operator_stack->data);
@@ -1062,7 +1153,6 @@ parse_expression (ArvEvaluator *evaluator)
 			status = ARV_EVALUATOR_STATUS_PARENTHESES_MISMATCH;
 			goto CLEANUP;
 		}
-
 		state.token_stack = g_slist_prepend (state.token_stack, state.operator_stack->data);
 		state.operator_stack = g_slist_delete_link (state.operator_stack, state.operator_stack);
 	}
@@ -1079,7 +1169,6 @@ parse_expression (ArvEvaluator *evaluator)
 	return evaluator->priv->rpn_stack == NULL ? ARV_EVALUATOR_STATUS_EMPTY_EXPRESSION : ARV_EVALUATOR_STATUS_SUCCESS;
 
 CLEANUP:
-
 	for (iter = state.garbage_stack; iter != NULL; iter = iter->next)
 		arv_evaluator_token_free (iter->data);
 	g_slist_free (state.garbage_stack);
@@ -1131,6 +1220,7 @@ arv_evaluator_evaluate_as_double (ArvEvaluator *evaluator, GError **error)
 	}
 
 	status = evaluate (evaluator->priv->rpn_stack, evaluator->priv->variables, NULL, &value);
+
 	if (status != ARV_EVALUATOR_STATUS_SUCCESS) {
 		arv_evaluator_set_error (error, status);
 		return 0.0;
@@ -1162,7 +1252,9 @@ arv_evaluator_evaluate_as_int64 (ArvEvaluator *evaluator, GError **error)
 	}
 
 	status = evaluate (evaluator->priv->rpn_stack, evaluator->priv->variables, &value, NULL);
+
 	if (status != ARV_EVALUATOR_STATUS_SUCCESS) {
+
 		arv_evaluator_set_error (error, status);
 		return 0.0;
 	}
@@ -1405,11 +1497,17 @@ arv_evaluator_class_init (ArvEvaluatorClass *evaluator_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (evaluator_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (evaluator_class, sizeof (ArvEvaluatorPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (evaluator_class);
 
 	object_class->finalize = arv_evaluator_finalize;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvEvaluator, arv_evaluator, G_TYPE_OBJECT)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvEvaluator, arv_evaluator, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvEvaluator))
+#endif
