@@ -74,25 +74,25 @@ arv_fake_device_get_genicam (ArvDevice *device)
 }
 
 static gboolean
-arv_fake_device_read_memory (ArvDevice *device, guint32 address, guint32 size, void *buffer, GError **error)
+arv_fake_device_read_memory (ArvDevice *device, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	return arv_fake_camera_read_memory (ARV_FAKE_DEVICE (device)->priv->camera, address, size, buffer);
 }
 
 static gboolean
-arv_fake_device_write_memory (ArvDevice *device, guint32 address, guint32 size, void *buffer, GError **error)
+arv_fake_device_write_memory (ArvDevice *device, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	return arv_fake_camera_write_memory (ARV_FAKE_DEVICE (device)->priv->camera, address, size, buffer);
 }
 
 static gboolean
-arv_fake_device_read_register (ArvDevice *device, guint32 address, guint32 *value, GError **error)
+arv_fake_device_read_register (ArvDevice *device, guint64 address, guint32 *value, GError **error)
 {
 	return arv_fake_camera_read_register (ARV_FAKE_DEVICE (device)->priv->camera, address, value);
 }
 
 static gboolean
-arv_fake_device_write_register (ArvDevice *device, guint32 address, guint32 value, GError **error)
+arv_fake_device_write_register (ArvDevice *device, guint64 address, guint32 value, GError **error)
 {
 	return arv_fake_camera_write_register (ARV_FAKE_DEVICE (device)->priv->camera, address, value);
 }
@@ -154,7 +154,9 @@ arv_fake_device_class_init (ArvFakeDeviceClass *fake_device_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (fake_device_class);
 	ArvDeviceClass *device_class = ARV_DEVICE_CLASS (fake_device_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (fake_device_class, sizeof (ArvFakeDevicePrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (fake_device_class);
 
@@ -169,4 +171,8 @@ arv_fake_device_class_init (ArvFakeDeviceClass *fake_device_class)
 	device_class->write_register = arv_fake_device_write_register;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvFakeDevice, arv_fake_device, ARV_TYPE_DEVICE)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvFakeDevice, arv_fake_device, ARV_TYPE_DEVICE, G_ADD_PRIVATE (ArvFakeDevice))
+#endif
