@@ -23,6 +23,10 @@
 #ifndef ARV_STREAM_H
 #define ARV_STREAM_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <arvtypes.h>
 
 G_BEGIN_DECLS
@@ -65,6 +69,8 @@ struct _ArvStream {
 struct _ArvStreamClass {
 	GObjectClass parent_class;
 
+	void		(*start_thread)		(ArvStream *stream);
+	void		(*stop_thread)		(ArvStream *stream);
 	void		(*get_statistics)	(ArvStream *stream, guint64 *n_completed_buffers,
 						 guint64 *n_failures, guint64 *n_underruns);
 
@@ -81,6 +87,9 @@ ArvBuffer * 	arv_stream_timeout_pop_buffer 		(ArvStream *stream, guint64 timeout
 void 		arv_stream_get_n_buffers 		(ArvStream *stream,
 							 gint *n_input_buffers,
 							 gint *n_output_buffers);
+void		arv_stream_start_thread			(ArvStream *stream);
+unsigned int	arv_stream_stop_thread			(ArvStream *stream, gboolean delete_buffers);
+
 void		arv_stream_get_statistics		(ArvStream *stream,
 							 guint64 *n_completed_buffers,
 							 guint64 *n_failures,
