@@ -398,7 +398,7 @@ arv_uv_device_write_register (ArvDevice *device, guint64 address, guint32 value,
 	return arv_uv_device_write_memory (device, address, sizeof (guint32), &value, error);
 }
 
-static void
+static gboolean
 _bootstrap (ArvUvDevice *uv_device)
 {
 	ArvDevice *device = ARV_DEVICE (uv_device);
@@ -427,17 +427,26 @@ _bootstrap (ArvUvDevice *uv_device)
 	GString *string;
 	void *data;
 	char manufacturer[64];
+	gboolean success = TRUE;
 
 	arv_debug_device ("Get genicam");
 
-	arv_device_read_memory(device, ARV_ABRM_MANUFACTURER_NAME, 64, &manufacturer, NULL);
+	success = success && arv_device_read_memory(device, ARV_ABRM_MANUFACTURER_NAME, 64, &manufacturer, NULL);
+	if (!success) {
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		return FALSE;
+	}
 	manufacturer[63] = 0;
 	arv_debug_device ("MANUFACTURER_NAME =        '%s'", manufacturer);
 
-	arv_device_read_memory (device, ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
-	arv_device_read_memory (device, ARV_ABRM_MAX_DEVICE_RESPONSE_TIME, sizeof (guint32), &response_time, NULL);
-	arv_device_read_memory (device, ARV_ABRM_DEVICE_CAPABILITY, sizeof (guint64), &device_capability, NULL);
-	arv_device_read_memory (device, ARV_ABRM_MANIFEST_TABLE_ADDRESS, sizeof (guint64), &manifest_table_address, NULL);
+	success = success && arv_device_read_memory (device, ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
+	success = success && arv_device_read_memory (device, ARV_ABRM_MAX_DEVICE_RESPONSE_TIME, sizeof (guint32), &response_time, NULL);
+	success = success && arv_device_read_memory (device, ARV_ABRM_DEVICE_CAPABILITY, sizeof (guint64), &device_capability, NULL);
+	success = success && arv_device_read_memory (device, ARV_ABRM_MANIFEST_TABLE_ADDRESS, sizeof (guint64), &manifest_table_address, NULL);
+	if (!success) {
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		return FALSE;
+	}
 
 	arv_debug_device ("MAX_DEVICE_RESPONSE_TIME = 0x%08x", response_time);
 	arv_debug_device ("DEVICE_CAPABILITY        = 0x%016lx", device_capability);
@@ -446,10 +455,14 @@ _bootstrap (ArvUvDevice *uv_device)
 
 	uv_device->priv->timeout_ms = MAX (ARV_UVCP_DEFAULT_RESPONSE_TIME_MS, response_time);
 
-	arv_device_read_memory (device, offset + ARV_SBRM_U3VCP_CAPABILITY, sizeof (guint32), &u3vcp_capability, NULL);
-	arv_device_read_memory (device, offset + ARV_SBRM_MAX_CMD_TRANSFER, sizeof (guint32), &max_cmd_transfer, NULL);
-	arv_device_read_memory (device, offset + ARV_SBRM_MAX_ACK_TRANSFER, sizeof (guint32), &max_ack_transfer, NULL);
-	arv_device_read_memory (device, offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
+	success = success && arv_device_read_memory (device, offset + ARV_SBRM_U3VCP_CAPABILITY, sizeof (guint32), &u3vcp_capability, NULL);
+	success = success && arv_device_read_memory (device, offset + ARV_SBRM_MAX_CMD_TRANSFER, sizeof (guint32), &max_cmd_transfer, NULL);
+	success = success && arv_device_read_memory (device, offset + ARV_SBRM_MAX_ACK_TRANSFER, sizeof (guint32), &max_ack_transfer, NULL);
+	success = success && arv_device_read_memory (device, offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
+	if (!success) {
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		return FALSE;
+	}
 
 	arv_debug_device ("U3VCP_CAPABILITY =         0x%08x", u3vcp_capability);
 	arv_debug_device ("MAX_CMD_TRANSFER =         0x%08x", max_cmd_transfer);
@@ -459,17 +472,21 @@ _bootstrap (ArvUvDevice *uv_device)
 	uv_device->priv->cmd_packet_size_max = MIN (uv_device->priv->cmd_packet_size_max, max_cmd_transfer);
 	uv_device->priv->ack_packet_size_max = MIN (uv_device->priv->ack_packet_size_max, max_ack_transfer);
 
-	arv_device_read_memory (device, sirm_offset + ARV_SI_INFO, sizeof (si_info), &si_info, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_CONTROL, sizeof (si_control), &si_control, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_PAYLOAD_SIZE, sizeof (si_req_payload_size), &si_req_payload_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_LEADER_SIZE, sizeof (si_req_leader_size), &si_req_leader_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_TRAILER_SIZE, sizeof (si_req_trailer_size), &si_req_trailer_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_MAX_LEADER_SIZE, sizeof (si_max_leader_size), &si_max_leader_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_PAYLOAD_SIZE, sizeof (si_payload_size), &si_payload_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_PAYLOAD_COUNT, sizeof (si_payload_count), &si_payload_count, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_TRANSFER1_SIZE, sizeof (si_transfer1_size), &si_transfer1_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_TRANSFER2_SIZE, sizeof (si_transfer2_size), &si_transfer2_size, NULL);
-	arv_device_read_memory (device, sirm_offset + ARV_SI_MAX_TRAILER_SIZE, sizeof (si_max_trailer_size), &si_max_trailer_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_INFO, sizeof (si_info), &si_info, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_CONTROL, sizeof (si_control), &si_control, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_PAYLOAD_SIZE, sizeof (si_req_payload_size), &si_req_payload_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_LEADER_SIZE, sizeof (si_req_leader_size), &si_req_leader_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_TRAILER_SIZE, sizeof (si_req_trailer_size), &si_req_trailer_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_MAX_LEADER_SIZE, sizeof (si_max_leader_size), &si_max_leader_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_PAYLOAD_SIZE, sizeof (si_payload_size), &si_payload_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_PAYLOAD_COUNT, sizeof (si_payload_count), &si_payload_count, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_TRANSFER1_SIZE, sizeof (si_transfer1_size), &si_transfer1_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_TRANSFER2_SIZE, sizeof (si_transfer2_size), &si_transfer2_size, NULL);
+	success = success && arv_device_read_memory (device, sirm_offset + ARV_SI_MAX_TRAILER_SIZE, sizeof (si_max_trailer_size), &si_max_trailer_size, NULL);
+	if (!success) {
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		return FALSE;
+	}
 
 	arv_debug_device ("SI_INFO =                  0x%08x", si_info);
 	arv_debug_device ("SI_CONTROL =               0x%08x", si_control);
@@ -483,8 +500,12 @@ _bootstrap (ArvUvDevice *uv_device)
 	arv_debug_device ("SI_TRANSFER2_SIZE =        0x%08x", si_transfer2_size);
 	arv_debug_device ("SI_MAX_TRAILER_SIZE =      0x%08x", si_max_trailer_size);
 
-	arv_device_read_memory (device, manifest_table_address, sizeof (guint64), &manifest_n_entries, NULL);
-	arv_device_read_memory (device, manifest_table_address + 0x08, sizeof (entry), &entry, NULL);
+	success = success && arv_device_read_memory (device, manifest_table_address, sizeof (guint64), &manifest_n_entries, NULL);
+	success = success && arv_device_read_memory (device, manifest_table_address + 0x08, sizeof (entry), &entry, NULL);
+	if (!success) {
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		return FALSE;
+	}
 
 	arv_debug_device ("MANIFEST_N_ENTRIES =       0x%016lx", manifest_n_entries);
 
@@ -497,7 +518,12 @@ _bootstrap (ArvUvDevice *uv_device)
 	arv_debug_device ("genicam size    =          0x%016lx", entry.size);
 
 	data = g_malloc0 (entry.size);
-	arv_device_read_memory (device, entry.address, entry.size, data, NULL);
+	success = success && arv_device_read_memory (device, entry.address, entry.size, data, NULL);
+	if (!success){
+		arv_warning_device ("[UvDevice::_bootstrap] Error during memory read");
+		g_free(data);
+		return FALSE;
+	}
 
 #if 0
 	string = g_string_new ("");
@@ -561,6 +587,7 @@ _bootstrap (ArvUvDevice *uv_device)
 	arv_debug_device("GENICAM\n:%s", uv_device->priv->genicam_xml);
 #endif
 
+	return TRUE;
 }
 
 static ArvGc *
@@ -701,7 +728,10 @@ arv_uv_device_new (const char *vendor, const char *product, const char *serial_n
 		return NULL;
 	}
 
-	_bootstrap (uv_device);
+	if ( !_bootstrap (uv_device)){
+		arv_warning_device ("[UvDevice::new] Failed to bootstrap USB device");
+		return NULL;
+	}
 
 	if (!ARV_IS_GC (uv_device->priv->genicam)) {
 		arv_warning_device ("[UvDevice::new] Failed to load genicam data");
