@@ -39,11 +39,6 @@
 
 static GObjectClass *parent_class = NULL;
 
-struct _ArvUvStreamPrivate {
-	GThread *thread;
-	void *thread_data;
-};
-
 /* Acquisition thread */
 
 typedef struct {
@@ -66,6 +61,11 @@ typedef struct {
 	guint n_underruns;
 } ArvUvStreamThreadData;
 
+struct _ArvUvStreamPrivate {
+	GThread *thread;
+	ArvUvStreamThreadData *thread_data;
+};
+
 static void *
 arv_uv_stream_thread (void *data)
 {
@@ -206,15 +206,21 @@ arv_uv_stream_thread (void *data)
 		}
 	}
 
+        if (buffer != NULL) {
+		buffer->priv->status = ARV_BUFFER_STATUS_ABORTED;
+		arv_stream_push_output_buffer (thread_data->stream, buffer);
+		if (thread_data->callback != NULL)
+			thread_data->callback (thread_data->user_data,
+					       ARV_STREAM_CALLBACK_TYPE_BUFFER_DONE,
+					       buffer);
+	}
+
 	if (thread_data->callback != NULL)
 		thread_data->callback (thread_data->user_data, ARV_STREAM_CALLBACK_TYPE_EXIT, NULL);
 
 	g_free (incoming_buffer);
 
         /* The thread was cancelled with unprocessed frame. Release it to prevent memory leak */
-        if (buffer != NULL)
-       		g_object_unref(buffer);
-
 	arv_log_stream_thread ("Stop USB3Vision stream thread");
 
 	return NULL;
@@ -230,22 +236,12 @@ align (guint32 val, guint32 alignment)
 	return (val + (alignment - 1)) & ~(alignment - 1);
 }
 
-/**
- * arv_uv_stream_new: (skip)
- * @uv_device: a #ArvUvDevice
- * @callback: (scope call): image processing callback
- * @user_data: (closure): user data for @callback
- *
- * Return Value: (transfer full): a new #ArvStream.
- */
-
-ArvStream *
-arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *user_data)
+static void
+arv_uv_stream_start_thread (ArvStream *stream)
 {
-	ArvDevice *device;
-	ArvUvStream *uv_stream;
+	ArvUvStream *uv_stream = ARV_UV_STREAM (stream);
 	ArvUvStreamThreadData *thread_data;
-	ArvStream *stream;
+	ArvDevice *device;
 	guint64 offset;
 	guint64 sirm_offset;
 	guint32 si_info;
@@ -260,9 +256,12 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	guint32 alignment;
 	guint32 aligned_maximum_transfer_size;
 
-	g_return_val_if_fail (ARV_IS_UV_DEVICE (uv_device), NULL);
+	g_return_if_fail (uv_stream->priv->thread == NULL);
+	g_return_if_fail (uv_stream->priv->thread_data != NULL);
 
-	device = ARV_DEVICE (uv_device);
+	thread_data = uv_stream->priv->thread_data;
+
+	device = ARV_DEVICE (thread_data->uv_device);
 
 	arv_device_read_memory (device, ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
 	arv_device_read_memory (device, offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
@@ -318,6 +317,61 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	si_control = 0x1;
 	arv_device_write_memory (device, sirm_offset + ARV_SI_CONTROL, sizeof (si_control), &si_control, NULL);
 
+	thread_data->leader_size = si_req_leader_size;
+	thread_data->payload_size = si_payload_size;
+	thread_data->trailer_size = si_req_trailer_size;
+	thread_data->cancel = FALSE;
+
+	uv_stream->priv->thread = g_thread_new ("arv_uv_stream", arv_uv_stream_thread, uv_stream->priv->thread_data);
+}
+
+static void
+arv_uv_stream_stop_thread (ArvStream *stream)
+{
+	ArvUvStream *uv_stream = ARV_UV_STREAM (stream);
+	ArvUvStreamThreadData *thread_data;
+	guint64 offset;
+	guint64 sirm_offset;
+	guint32 si_control;
+
+	g_return_if_fail (uv_stream->priv->thread != NULL);
+	g_return_if_fail (uv_stream->priv->thread_data != NULL);
+
+	thread_data = uv_stream->priv->thread_data;
+
+	g_atomic_int_set (&uv_stream->priv->thread_data->cancel, TRUE);
+	g_thread_join (uv_stream->priv->thread);
+
+	uv_stream->priv->thread = NULL;
+
+	si_control = 0x0;
+	arv_device_read_memory (ARV_DEVICE (thread_data->uv_device),
+				ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
+	arv_device_read_memory (ARV_DEVICE (thread_data->uv_device),
+				offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
+	arv_device_write_memory (ARV_DEVICE (thread_data->uv_device),
+				 sirm_offset + ARV_SI_CONTROL, sizeof (si_control), &si_control, NULL);
+
+}
+
+/**
+ * arv_uv_stream_new: (skip)
+ * @uv_device: a #ArvUvDevice
+ * @callback: (scope call): image processing callback
+ * @user_data: (closure): user data for @callback
+ *
+ * Return Value: (transfer full): a new #ArvStream.
+ */
+
+ArvStream *
+arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *user_data)
+{
+	ArvUvStream *uv_stream;
+	ArvUvStreamThreadData *thread_data;
+	ArvStream *stream;
+
+	g_return_val_if_fail (ARV_IS_UV_DEVICE (uv_device), NULL);
+
 	uv_stream = g_object_new (ARV_TYPE_UV_STREAM, NULL);
 
 	stream = ARV_STREAM (uv_stream);
@@ -327,18 +381,14 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	thread_data->stream = stream;
 	thread_data->callback = callback;
 	thread_data->user_data = user_data;
-	thread_data->cancel = FALSE;
-
-	thread_data->leader_size = si_req_leader_size;
-	thread_data->payload_size = si_payload_size;
-	thread_data->trailer_size = si_req_trailer_size;
 
 	thread_data->n_completed_buffers = 0;
 	thread_data->n_failures = 0;
 	thread_data->n_underruns = 0;
 
 	uv_stream->priv->thread_data = thread_data;
-	uv_stream->priv->thread = g_thread_new ("arv_uv_stream", arv_uv_stream_thread, uv_stream->priv->thread_data);
+
+	arv_uv_stream_start_thread (ARV_STREAM (uv_stream));
 
 	return ARV_STREAM (uv_stream);
 }
@@ -372,25 +422,13 @@ arv_uv_stream_finalize (GObject *object)
 {
 	ArvUvStream *uv_stream = ARV_UV_STREAM (object);
 
-	if (uv_stream->priv->thread != NULL) {
+	arv_uv_stream_stop_thread (ARV_STREAM (uv_stream));
+
+	if (uv_stream->priv->thread_data != NULL) {
 		ArvUvStreamThreadData *thread_data;
-		guint64 offset;
-		guint64 sirm_offset;
-		guint32 si_control;
 
 		thread_data = uv_stream->priv->thread_data;
 
-		g_atomic_int_set (&thread_data->cancel, TRUE);
-		g_thread_join (uv_stream->priv->thread);
-
-		si_control = 0x0;
-		arv_device_read_memory (ARV_DEVICE (thread_data->uv_device),
-					ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
-		arv_device_read_memory (ARV_DEVICE (thread_data->uv_device),
-					offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
-		arv_device_write_memory (ARV_DEVICE (thread_data->uv_device),
-					 sirm_offset + ARV_SI_CONTROL, sizeof (si_control), &si_control, NULL);
-
 		arv_debug_stream ("[UvStream::finalize] n_completed_buffers    = %u",
 				  thread_data->n_completed_buffers);
 		arv_debug_stream ("[UvStream::finalize] n_failures             = %u",
@@ -399,10 +437,7 @@ arv_uv_stream_finalize (GObject *object)
 				  thread_data->n_underruns);
 
 		g_clear_object (&thread_data->uv_device);
-		g_free (thread_data);
-
-		uv_stream->priv->thread_data = NULL;
-		uv_stream->priv->thread = NULL;
+		g_clear_pointer (&uv_stream->priv->thread_data, g_free);
 	}
 
 	parent_class->finalize (object);
@@ -422,6 +457,8 @@ arv_uv_stream_class_init (ArvUvStreamClass *uv_stream_class)
 
 	object_class->finalize = arv_uv_stream_finalize;
 
+	stream_class->start_thread = arv_uv_stream_start_thread;
+	stream_class->stop_thread = arv_uv_stream_stop_thread;
 	stream_class->get_statistics = arv_uv_stream_get_statistics;
 }
 
